
;****************************************************
;
;******************* FREEZER TRANS *****************
;
;
;****************************************************

; 02/26/2011 
; USING A/D CONVERTER FOR 1/2 VOLTAGE
; WRITTEN BY RON NEWTON

	RADIX		HEX	;HEX IS DEFAULT # FORMAT
	LIST      P=PIC12F675  ; LIST DIRECTIVE TO DEFINE PROCESSOR
	INCLUDE "P12F675.INC"  ; PROCESSOR SPECIFIC VARIABLE DEFINITIONS
	
    __CONFIG _CP_OFF & _BODEN_OFF & _PWRTE_OFF & _MCLRE_OFF &_WDT_OFF & _INTRC_OSC_NOCLKOUT

;1 V+
;2 GP5			LINE
;3 GP4 			PDN
;4 GP3 			NC
;5 GP2			DATA_OUT
;6 GP1   		A/D
;7 GP0 			NC
;8 GND
   


     ORG       0H
     GOTO      AWAYWEGO
       

RESULTS		  	EQU	  	22H				;RESULTS REGISTER FOR TEMP AND POWER
KEY1		  	EQU	  	23H				;KEY FOR RECEIVER
XMT_BYTE		EQU		24H				;TRANSMIT REGISTER
XMT_BYTE1		EQU		25H				;SECOND TRANSMIT REGISTER
TEMP			EQU		26H				;TEMPORARY REGISTER 
DELAY_CNTR		EQU		27H				;DELAY COUNTER FOR SENDING
DELAY_CNTR1		EQU		28H				;SECOND DELAY COUNTER FOR SENDING
BIT_CNTR		EQU		29H				;SENDING COUNTER
BIT_CNTR1		EQU		2AH				;SECOND SENDING COUNTER
WAIT			EQU		2BH				;COUNTING REGISTER
AGAIN			EQU		2CH				;COUNTING REGISTER
COUNTER1		EQU		2DH				;TIMER REGISTER
COUNTER2		EQU		2EH				;TIMER REGISTER
COUNTER3		EQU		2FH				;TIMER REGISTER


#DEFINE		LINE	    GPIO,5			;LINE IN
#DEFINE		PDN			GPIO,4			;PDN OUT
#DEFINE		DATA_OUT	GPIO,2 			;DATA OUT


;**********************INITIALIZING**************************

AWAYWEGO
		MOVLW		0AAH		 		;LOAD KEY
		MOVWF		KEY1				;LOAD
        BSF   		STATUS, RP0        	;JUMP TO PAGE 1
	 	MOVLW		B'00100010'			;MAKING INPUT AND OUTPUTS
	 	MOVWF		TRISIO				;LOAD
		MOVLW		.0					;LOAD OPTION
        MOVWF		OPTION_REG		   	;LOAD
		MOVLW		B'00010010'			;LOADING A/D
		MOVWF		ANSEL        		;LOAD
		CLRF	   	INTCON				;NO INTERRUPTS
		MOVLW		B'00100000'			;LOAD WPU GP5 PULL UP
		MOVWF		WPU					;LOAD
       	BCF 	   	STATUS, RP0       	;BACK TO PAGE 0
		MOVLW		B'00000101'			;TURN ON A/D
		MOVWF		ADCON0				;LOAD
		CLRF		RESULTS				;CLEAR RESULTS

;******************TESTING AREA*********************
TESTING
	GOTO	S7A

;*******************STARTING AREA ***********************
START  	MOVF	TEMP,0				;CHECKING TO SEE A CHANGE OCCURED
		SUBWF	RESULTS,0			;SUBTRACT RESULTS FROM TEMP
		BTFSC	3,2					;CHECK FOR ZERO
		GOTO	S1					;NO CHANGE CHECK TEMP AND LINE
		MOVF	RESULTS,0			;CHANGE OCCURED LOAD TEMP FOR NEXT CHANGE
		MOVWF	TEMP				;LOAD
		GOTO	SENDING				;START SENDING

;*************TURNING ON A/D ****************
S1		BSF		ADCON0,1			;START A/D
		BTFSC	ADCON0,1			;WAIT UNTIL FINISHED
		GOTO	$-1

;***********ADDING HYSERISIS*******************		
		MOVLW	.128				;1/2 VOLTAGE IF BELOW
		SUBWF	ADRESH,0
		BTFSS	3,0
		GOTO	S2					;ABOVE JUMP TO S2
		BCF		RESULTS,0			;HIGH SET RESULTS,0 
		GOTO	LINE_CHECK			;JUMP TO LINE CHECK
S2		BSF		RESULTS,0			;LOW SET RESULTS0


LINE_CHECK							;LABEL
		BTFSS	LINE				;CHECKING LINE IF HIGH OR LOW
		GOTO	S4					;LINE LOW JUMP		
		BSF		RESULTS,1			;LINE HIGH SET RESULTS,1			
		GOTO	S5					;DO OVER
S4		BCF		RESULTS,1			;LINE LOW CLEAR RESULTS,1

S5		GOTO	START				;DO OVER

;******************SENDING AREA*********************

SENDING
		MOVLW	.10					; SEND 10 TIMES
		MOVWF	AGAIN				; LOAD
S7	    CALL	SEND			  	; SEND RESULTS
		DECFSZ	AGAIN,1				; SEND 10 TIMES
		GOTO	S7					; CHECK FOR 10
S7A		MOVLW	.80					; LOADING FOR 15 SECONDS
		MOVWF	COUNTER3			; LOAD
S8		DECFSZ	COUNTER1,1			; ANTI-CHATTER TIMER
		GOTO	S8
		DECFSZ	COUNTER2,1			; ANTI-CHATTER TIMER
		GOTO	S8
		DECFSZ	COUNTER3,1			; ANTI-CHATTER TIMER
		GOTO	S8
		GOTO	START				; GO BACK TO START
        
;********************RS232 TRANSMITTER INSTRUCTIONS***********
SEND 
		MOVF	KEY1,0				;LOAD KEY
		MOVWF	XMT_BYTE			;LOADING INTO XMT BYTE
		MOVF	RESULTS,0			;LOAD RESULTS
		MOVWF	XMT_BYTE1			;LOAD XMT BYTE1
		BSF		PDN					;TURN ON LINX
		BSF		DATA_OUT			;KEEP DATA IN OUT HIGH
		MOVLW	.16					;LOAD 16
		MOVWF	WAIT				;LOAD INTO WAIT
		CALL	BIT_DELAY			;KEEP HIGH FOR 16 BIT_DELAYS
		DECFSZ  WAIT,1				;DO 8 TIMES
		GOTO	$-2					;JUMP BACK 2
	  	MOVLW	D'8'			   	;LOAD 8 INTO W EIGHT BITS IN A BYTE
	    MOVWF	BIT_CNTR           	;LOAD
	    MOVWF	BIT_CNTR1		   	;LOAD
	    BCF	    DATA_OUT			;BRING DATA OUT LOW
        CALL    BIT_DELAY          	;START BIT SERIAL LOW FOR 1200 BAUD

XMIT    RRF     XMT_BYTE,1         ;ROTATE XMT BYTE FOR NEXT BIT
	    BTFSS	3,0				   ;CHECK TO SEE BIT SET
	    BCF		DATA_OUT		   ;NOT SET BRING SERIAL OUT LOW
	    BTFSC	3,0				   ;CHECK TO SEE IF SET	
	    BSF		DATA_OUT		   ;BIT SET MAKE SERIAL OUT HIGH
        CALL    BIT_DELAY          ;DATA BIT DELAY FOR 1200 BAUD
        DECFSZ 	BIT_CNTR,1		   ;CHECK TO SEE IT 8 BITS HAVE BEEN SENT
        GOTO	XMIT               ;DO UNITL EIGHT BITS ARE SENT

;********************SECOND BYTE BEING SENT*********************  
XMIT1	RRF     XMT_BYTE1,1         ;ROTATE XMT BYTE FOR NEXT BIT
	    BTFSS	3,0					;CHECK TO SEE BIT SET
	    BCF		DATA_OUT			;NOT SET BRING SERIAL OUT LOW
	    BTFSC	3,0  				;CHECK TO SEE IF SET	
	    BSF		DATA_OUT 			;BIT SET MAKE SERIAL OUT HIGH
        CALL    BIT_DELAY          	;DATA BIT DELAY FOR 1200 BAUD  
        DECFSZ 	BIT_CNTR1,1  		;CHECK TO SEE IT 8 BITS HAVE BEEN SENT
        GOTO	XMIT1               ;GET EIGHT BITS        
		BCF		PDN					;TURN OFF LINX
		RETURN
;********************RS232 INSTRUCTIONS***********

BIT_DELAY 
		MOVLW		D'200'		    ;SETUP FOR 1200 BAUD
		MOVWF		DELAY_CNTR      ;DELAY LOOP
LOOP_1  NOP							;DELAY HELP
		DECFSZ  	DELAY_CNTR,1	;DECREASING COUNTER		
		GOTO		LOOP_1    		;DECREASE COUNTER
		RETLW 	    0               ;RETURN TO CALL

START_DELAY							;LABEL
		MOVLW		D'100'		   	;SETUP FOR 1200 BAUD SETS FOR DETECTING MIDDLE OF BIT
		MOVWF		DELAY_CNTR	   	;DELAY LOOP
		
LOOP_2	NOP							;DELAY HELP
		DECFSZ		DELAY_CNTR,1	;DECREASEING COUNTER
	    GOTO		LOOP_2          ;DECREASE COUNTER
        RETLW 		0              	;RETURN TO CALL

		  END                      	;END OF PROGRAM

